@@ -1,21 +1,31 @@
-from jenkinsapi.jenkinsbase import JenkinsBase
-from jenkinsapi.fingerprint import Fingerprint
-from jenkinsapi.job import Job
-from jenkinsapi.view import View
-from jenkinsapi.node import Node
 from jenkinsapi.exceptions import UnknownJob, NotAuthorized
-from utils.urlopener import mkurlopener, mkkrbopener, mkopener, NoAuto302Handler
+from jenkinsapi.fingerprint import Fingerprint
+from jenkinsapi.jenkinsbase import JenkinsBase
+from jenkinsapi.job import Job
+from jenkinsapi.node import Node
+from jenkinsapi.queue import Queue
+from jenkinsapi.view import View
+from jenkinsapi import config
+from utils.urlopener import mkurlopener, mkopener, NoAuto302Handler
+import cookielib
 import logging
 import time
-import urllib2
 import urllib
+import urllib2
 import urlparse
-import cookielib
+
 try:
     import json
 except ImportError:
     import simplejson as json
 
+try:
+    # Kerberos is now an extras_require - please see
+    # http://pythonhosted.org/distribute/setuptools.html#declaring-extras-optional-features-with-their-own-dependencies
+    from utils.urlopener_kerberos import mkkrbopener
+except ImportError:
+    mkkrbopener = None
+
 log = logging.getLogger(__name__)
 
 class Jenkins(JenkinsBase):
@@ -61,6 +71,9 @@ class Jenkins(JenkinsBase):
         log.debug("args: %s" % auth_args)
         return auth_args
 
+    def get_base_server_url(self):
+        return self.baseurl[:-(len(config.JENKINS_API))] 
+
     def get_opener(self):
         if self.formauth:
             return self.get_login_opener()
@@ -78,6 +91,8 @@ class Jenkins(JenkinsBase):
         return mkopener(*hdrs)
 
     def get_krb_opener(self):
+        if not mkkrbopener:
+            raise NotImplementedError('JenkinsAPI was installed without Kerberos support.')
         return mkkrbopener(self.baseurl)
 
     def login(self):
@@ -303,7 +318,10 @@ class Jenkins(JenkinsBase):
         """
         for url, name in self.get_jobs_info():
             if name == jobname:
-                return Job(url, name, jenkins_obj=self)
+                preferred_scheme = urlparse.urlsplit(self.baseurl).scheme
+                url_split = urlparse.urlsplit(url)
+                preferred_url = urlparse.urlunsplit([preferred_scheme, url_split.netloc, url_split.path, url_split.query, url_split.fragment])
+                return Job(preferred_url, name, jenkins_obj=self)
         raise UnknownJob(jobname)
 
     def get_node_dict(self):
@@ -324,6 +342,14 @@ class Jenkins(JenkinsBase):
         url = "%(baseurl)s/computer/%(nodename)s" % {'baseurl': self.baseurl, 'nodename': urllib.quote(nodename)}
         return url
 
+    def get_queue_url(self):
+        url = "%(baseurl)s/queue/" % {'baseurl': self.get_base_server_url()}
+        return url
+
+    def get_queue(self):
+        queue_url = self.get_queue_url()
+        return Queue(queue_url, self)
+
     def has_node(self, nodename):
         """
         Does a node by the name specified exist
