@@ -9,7 +9,7 @@ from jenkinsapi.build import Build
 from jenkinsapi.jenkinsbase import JenkinsBase
 from jenkinsapi import exceptions
 
-from exceptions import NoBuildData, NotFound
+from exceptions import NoBuildData, NotFound, NotInQueue
 
 log = logging.getLogger(__name__)
 
@@ -64,21 +64,23 @@ class Job(JenkinsBase):
             self._element_tree = ET.fromstring(self._config)
         return self._element_tree
 
-    def get_build_triggerurl( self, token=None, params={} ):
-        if token is None and not params:
+    def get_build_triggerurl(self, token=None, params=None):
+        if token is None and params is None:
             extra = "build"
         elif params:
             if token:
                 assert isinstance(token, str ), "token if provided should be a string."
                 params['token'] = token
-            extra = "buildWithParameters?" + urllib.urlencode(params)
+            extra = "buildWithParameters"
         else:
             assert isinstance(token, str ), "token if provided should be a string."
-            extra = "build?" + urllib.urlencode({'token':token})
+            params = dict()
+            params['token'] = token
+            extra = "build"
         buildurl = urlparse.urljoin( self.baseurl, extra )
-        return buildurl
+        return buildurl, params
 
-    def invoke(self, securitytoken=None, block=False, skip_if_running=False, invoke_pre_check_delay=3, invoke_block_delay=15, params={}):
+    def invoke(self, securitytoken=None, block=False, skip_if_running=False, invoke_pre_check_delay=3, invoke_block_delay=15, params=None):
         assert isinstance( invoke_pre_check_delay, (int, float) )
         assert isinstance( invoke_block_delay, (int, float) )
         assert isinstance( block, bool )
@@ -94,8 +96,8 @@ class Job(JenkinsBase):
                 log.warn("Will re-schedule %s even though it is already running" % self.id() )
         original_build_no = self.get_last_buildnumber()
         log.info( "Attempting to start %s on %s" % ( self.id(), repr(self.get_jenkins_obj()) ) )
-        url = self.get_build_triggerurl( securitytoken, params)
-        html_result = self.hit_url(url)
+        url, params = self.get_build_triggerurl( securitytoken, params)
+        html_result = self.hit_url(url, params)
         assert len( html_result ) > 0
         if invoke_pre_check_delay > 0:
             log.info("Waiting for %is to allow Jenkins to catch up" % invoke_pre_check_delay )
@@ -325,7 +327,7 @@ class Job(JenkinsBase):
     def update_config(self, config):
         """
         Update the config.xml to the job
-        Also refresh the BeautifulSoup object since the config has changed
+        Also refresh the ElementTree object since the config has changed
         """
         post_data = self.post_data("%(baseurl)s/config.xml" % self.__dict__, config)
         self._element_tree = ET.fromstring(config)
@@ -385,10 +387,55 @@ class Job(JenkinsBase):
 
     def disable(self):
         '''Disable job'''
-        disableurl = urlparse.urljoin( self.baseurl, 'disable' )
+        disableurl = urlparse.urljoin(self.baseurl, 'disable' )
         return self.post_data(disableurl, '')
 
     def enable(self):
         '''Enable job'''
-        enableurl = urlparse.urljoin( self.baseurl, 'enable' )
+        enableurl = urlparse.urljoin(self.baseurl, 'enable' )
         return self.post_data(enableurl, '')
+
+    def delete_from_queue(self):
+        """
+        Delete a job from the queue only if it's enqueued
+        :raise NotInQueue if the job is not in the queue
+        """
+        if not self.is_queued():
+            raise NotInQueue()
+        queue_id = self._data['queueItem']['id']
+        cancelurl = urlparse.urljoin(self.get_jenkins_obj().get_queue().baseurl,
+                                     'cancelItem?id=%s' % queue_id)
+        try:
+            self.post_data(cancelurl, '')
+        except urllib2.HTTPError:
+            # The request doesn't have a response, so it returns 404,
+            # it's the expected behaviour
+            pass
+        return True
+
+    def get_params(self):
+        """
+        Get the parameters for this job. Format varies by parameter type. Here
+        is an example string parameter:
+            {
+                'type': 'StringParameterDefinition',
+                'description': 'Parameter description',
+                'defaultParameterValue': {'value': 'default value'},
+                'name': 'FOO_BAR'
+            }
+        """
+        for action in self._data['actions']:
+            try:
+                for param in action['parameterDefinitions']:
+                    yield param
+            except KeyError:
+                continue
+
+    def get_params_list(self):
+        """
+        Gets the list of parameter names for this job.
+        """
+        params = []
+        for param in self.get_params():
+            params.append(param['name'])
+        return params
